@@ -138,6 +138,7 @@ bonobo_window_dispose (GObject *object)
 	BonoboWindow *win = (BonoboWindow *)object;
 	
 	if (win->priv->engine) {
+		bonobo_ui_engine_dispose (win->priv->engine);
 		g_object_unref (G_OBJECT (win->priv->engine));
 		win->priv->engine = NULL;
 	}
@@ -208,6 +209,7 @@ construct_priv (BonoboWindow *win)
 #ifdef FIXME
 	gtk_menu_bar_set_shadow_type (GTK_MENU_BAR (priv->menu), GTK_SHADOW_NONE);
 #endif
+#if 0
 	if (bonobo_ui_preferences_get_menubar_relief ()) {
 		guint border_width;
 
@@ -217,7 +219,7 @@ construct_priv (BonoboWindow *win)
 			border_width -= 2;
 		gtk_container_set_border_width (GTK_CONTAINER (priv->menu), border_width);
 	} else
-		bonobo_dock_item_set_shadow_type (BONOBO_DOCK_ITEM (priv->menu_item), GTK_SHADOW_NONE);
+#endif
 
 	priv->main_vbox = gtk_vbox_new (FALSE, 0);
 	bonobo_dock_set_client_area (priv->dock, priv->main_vbox);
